// GLWVideo.cpp: implementation of the GLWVideo class.
//
//////////////////////////////////////////////////////////////////////

#include <windows.h>		// File d'Intestazione per Windows
#include <math.h>			// File d'Intestazione per la libreria Matematica
#include <stdio.h>			// File d'Intestazione per Standard Input/Output
#include <gl\gl.h>			// File d'Intestazione per la libreria OpenGL32 
#include <gl\glu.h>			// File d'Intestazione per la libreria GLu32 
#include <gl\glaux.h>		// File d'Intestazione per la libreria Glaux 
#include "JPEG.h"           // Funzioni per il caricamento di texture JPEG

#include "GLWVideo.h"

/* Variabili chiave dell'applicazione */
	extern HDC			hDC;		// Device Context GDI privato
	extern HGLRC		hRC;		// Rendering Context permanente
	extern HWND			hWnd;		// Handle alla Finestra 
	extern HINSTANCE	hInstance;		// Istanza dell'Applicazione


	extern bool	keys[256];			// Array usato per la routine della tastiera
	extern bool	active;		// Flag di Finestra Attiva, settato a TRUE di default

/* Movimento del soggetto */
	extern float	heading;
	extern float	xpos;
	extern float	zpos;
	extern float	ypos;
	extern int		vel_gra;

	extern GLfloat	yrot;				// Rotazione Y 
	extern GLfloat  walkbias;
	extern GLfloat  walkbiasangle;
	extern GLfloat  lookupdown;
	extern GLfloat	z;				    // Profondit nello Schermo

/* Costanti matematiche */
	const float piover180 = 0.0174532925f;

/* variabili di gioco */
	extern int vittoria;  // 0 niente, -1 sconfitta, 1 vittoria (l'uscita  considerata come sconfitta */

/* Definizioni di strutture */
	struct TEXTURE
	{
		/*GLuint*/UINT texture;			// Spazio per Texture
		TEXTURE *next;
		char nome_file[20];     // nome_file, serve per non inserire texture doppione
	};

	TEXTURE	*texture_testa, *texture_pos;			// Lista di Texture // inserir gli elementi in coda, per facilitarne la ricerca
	typedef struct tagVERTEX
	{
		float x, y, z;
		float u, v;
	} VERTEX;

	typedef struct tagTRIANGLE
	{
		VERTEX vertex[3];
		int n_texture;           // indice della texture interessata
	} TRIANGLE;

	typedef struct tagSECTOR
	{
		int numtriangles;
		TRIANGLE* triangle;
	} SECTOR;

/* fps */
	BOOL fpsActive; // se visualizzo a video lo stato attuale della variabile fps
	extern int fps;        // devono essere 60
	extern int ral;

SECTOR sector1;				// Il nostro modello world andr qui:

/* Collisioni */
	extern int collidono;
	extern int suolo;

extern LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Prototipo di funzione per WndProc

//////////////////////////////////////////////////////////////////////
// class GLWVideo
//////////////////////////////////////////////////////////////////////

GLWVideo::GLWVideo()
{
	
};

GLWVideo::~GLWVideo()
{

};


GLWVideo::GLWVideo(float ral, int x, int y, int bits)
{
	fullscreen=TRUE;

	Rallentamento=ral;
	Screen_X=x;
	Screen_Y=y;
	Screen_BITS=bits;

	walkbias = 0;
	walkbiasangle = 0;
	lookupdown = 0.0f;
	z=0.0f;	
	
	avanti=0;
	max_avanti=10;
	giu=0;
	ypos=0;

	vittoria=0;

	fpsActive=false;
	fps=0;
};

void GLWVideo::readstr(FILE *f,char *string)
{
	do
	{
		fgets(string, 255, f);
	} while ((string[0] == '/') || (string[0] == '\n'));   // vado a capo finch ho commenti o righe vuote
	return;
};

AUX_RGBImageRec *GLWVideo::LoadBMP(char *Filename)				// Carica un'immagine Bitmap 
{
	FILE *File=NULL;									// Handle a File 

	if (!Filename)										// Si assicura che il nome di file sia stato indicato
    {
		return NULL;									// Se no, restituisce NULL
    }

	File=fopen(Filename,"r");							// Test per vedere se il file esiste
        
	if (File)											// Il file esiste?
	{
		fclose(File);									// Chiude l'Handle
		return auxDIBImageLoad(Filename);				// Carica la Bitmap e restituisce un puntatore
	}

	return NULL;										// Se il caricamento fallisce restituisce NULL
};

int GLWVideo::LoadGLTextures()   // inizializza le variabili relative alle textures
{	
	texture_testa=NULL;
	texture_pos=NULL;

	return TRUE;
};

int GLWVideo::AddTexture(char *nome_file)
{
	int Status=FALSE;									// Indicatore dello stato 
	TEXTURE *tNew;

	AUX_RGBImageRec *TextureImage[1];					// Crea spazio per la texture

	memset(TextureImage,0,sizeof(void *)*1);           	// Setta il puntatore a NULL

	char ff[255];
	strcpy(ff, "data/textures/");  // directory di default
	strcpy(&ff[14], nome_file);
	
	/* gestione lista */
		tNew=new TEXTURE;
		tNew->next=NULL;
		if (texture_testa==NULL)       // lista non molto "raffinata", ma efficace
			texture_testa=tNew;
		else
			texture_pos->next=tNew;	
		texture_pos=tNew;

	// Carica la JPEG o Bitmap, testa per errori, se la non viene trovato il file chiude l'applicazione
	if (TextureImage[0]=LoadBMP(ff))  
	{
		Status=TRUE;							// Setta la variabile Status a TRUE
		
		glGenTextures(1, &tNew->texture);          // Crea la texture

		// Crea una Texture MipMapped 
		glBindTexture(GL_TEXTURE_2D, tNew->texture);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}
	else if (JPEG_Texture(&tNew->texture, ff)==1)
	{		
		Status=TRUE;		
	}

    if (TextureImage[0])						// Se la texture esiste
	{
		if (TextureImage[0]->data)              // Se l'immagine texture esiste
        {
			free(TextureImage[0]->data);        // Libera la memoria per l'immagine Texture 
        }
		
		free(TextureImage[0]);                  // Libera la struttura immagine
        
	}
	strcpy(tNew->nome_file, nome_file);         // la texture deve mantenere un riferimento al file di origine

	return Status;                              // Restituisce la variabile Status
};

void GLWVideo::SetupWorld()
{	
	float x, y, z, u, v;
	int numtriangles, i;
	TEXTURE *tPos;
	FILE *filein;
	char oneline[255];
	char nome_file[20];
	filein = fopen("data/world.txt", "rt");				// File da cui caricare i dati del mondo

	readstr(filein,oneline);
	sscanf(oneline, "NUMPOLLIES %d\n", &numtriangles);

	sector1.triangle = new TRIANGLE[numtriangles];
	sector1.numtriangles = numtriangles;
	for (int loop = 0; loop < numtriangles; loop++) // legge tutti i dati del file
	{
		for (int vert = 0; vert < 3; vert++)        // legge i tre vertici del triangolo
		{
			readstr(filein,oneline);
			sscanf(oneline, "%f %f %f %f %f", &x, &y, &z, &u, &v);
			sector1.triangle[loop].vertex[vert].x = -x;
			sector1.triangle[loop].vertex[vert].y = y;
			sector1.triangle[loop].vertex[vert].z = z;
			sector1.triangle[loop].vertex[vert].u = u;
			sector1.triangle[loop].vertex[vert].v = v;
		}
		// leggo il nome_file della texture, se specificato
		 if (fgets( oneline, 256, filein)==NULL)	return; 
		 if (oneline[0]=='#') // c' una texture
		 {			
			oneline[strlen(oneline)-1]='\0';
			strcpy(nome_file, &oneline[1]);
			// verifico se la texture  gi stata caricata
				for (i=0, tPos=texture_testa; (tPos!=NULL) && (strcmp(tPos->nome_file, nome_file)); i++, tPos=tPos->next);
				sector1.triangle[loop].n_texture=i; // in ogni caso, i indica l'indice della texture interesata
				
				if (tPos==NULL) // si deve creare la nuova texture
				{
					AddTexture(nome_file);
				}
			// altrimenti la carico e assegno il nuovo indice al sector1.triangle[loop].n_texture
		 }
		 else 
			 	sector1.triangle[loop].n_texture=-1;
	}
	fclose(filein);

	InitTextureVittoria();

	return;
};

GLvoid GLWVideo::InitTextureVittoria(GLvoid)
{
	TEXTURE *tPos;
	int i;
	for (i=0, tPos=texture_testa; (tPos!=NULL) && (strcmp(tPos->nome_file, "FINISH.BMP")); i++, tPos=tPos->next)
		;
	
	texture_vittoria=i;
}

GLvoid GLWVideo::ReSizeGLScene(GLsizei width, GLsizei height)		// Ridimensiona e Inizializza la Finestra GL 
{
	if (height==0)										// Evita una divisione per zero
	{
		height=1;										// Imposta l'Altezza a Uno
	}

	glViewport(0,0,width,height);						// Resetta la Viewport corrente

	glMatrixMode(GL_PROJECTION);						// Seleziona la Projection Matrix
	glLoadIdentity();									// Resetta la Projection Matrix

	// Calcola l'Aspect Ratio della finestra
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);							// Seleziona la Modelview Matrix
	glLoadIdentity();									// Resetta la Modelview Matrix
};

int GLWVideo::InitGL(GLvoid)										// Tutti i settaggi per OpenGL vengono fatti qui
{
	if (!LoadGLTextures())								// Salta alla routine di caricamento Texture 
	{
		return FALSE;									// Se la Texture non  caricata restituisce FALSE
	}

	glEnable(GL_TEXTURE_2D);							// Abilita il Texture Mapping 
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Setta la funzione di Blending per la traslucenza
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Colore di sfondo Nero
	glClearDepth(1.0f);									// Setta il Depth Buffer 
	glDepthFunc(GL_LESS);								// Il tipo di Depth Test da fare
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glShadeModel(GL_SMOOTH);							// Abilita lo Smooth Shading
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Calcoli per la Prospettiva ideali

	BuildFont();  // inizializzazione dei caratteri per la scrittura di testo

	SetupWorld();

	return TRUE;										// Inizializzazione andata a buon fine
};

int GLWVideo::DrawGLScene(GLvoid)									// Qui  dove farete tutti i disegni
{
	if (!fpsActive) 	
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Pulisce lo schermo e il Depth Buffer	
	
	glLoadIdentity();									// Resetta la View

	GLfloat x_m, y_m, z_m, u_m, v_m;
	GLfloat xtrans = -xpos;
	GLfloat ztrans = -zpos;
	GLfloat ytrans = -walkbias-0.25f-ypos;
	GLfloat sceneroty = 360.0f - yrot;

	int numtriangles, i;

	glRotatef(lookupdown,1.0f,0,0);
	glRotatef(sceneroty,0,1.0f,0);
	
	glTranslatef(xtrans, ytrans, ztrans);
	//glBindTexture(GL_TEXTURE_2D, texture_pos->texture);
	
	numtriangles = sector1.numtriangles;
	
	// Elabora ogni Triangolo
	glColor3ub(255, 255, 255); // colori texture originali

	for (int loop_m = 0; loop_m < numtriangles; loop_m++)
	{
		if (sector1.triangle[loop_m].n_texture!=-1)  // se c' una texture la carico
		{
			for (i=0, texture_pos=texture_testa; i<sector1.triangle[loop_m].n_texture; i++, texture_pos=texture_pos->next);    // cerco la texture interessata
			glBindTexture(GL_TEXTURE_2D, texture_pos->texture);
		}

		glBegin(GL_TRIANGLES);   // tutto questo dovr essere modificato per caricare una lista di triangoli
			glNormal3f( 0.0f, 0.0f, 1.0f);
			x_m = sector1.triangle[loop_m].vertex[0].x;
			y_m = sector1.triangle[loop_m].vertex[0].y;
			z_m = sector1.triangle[loop_m].vertex[0].z;
			u_m = sector1.triangle[loop_m].vertex[0].u;
			v_m = sector1.triangle[loop_m].vertex[0].v;
			glTexCoord2f(u_m,v_m); glVertex3f(x_m,y_m,z_m);
			
			x_m = sector1.triangle[loop_m].vertex[1].x;
			y_m = sector1.triangle[loop_m].vertex[1].y;
			z_m = sector1.triangle[loop_m].vertex[1].z;
			u_m = sector1.triangle[loop_m].vertex[1].u;
			v_m = sector1.triangle[loop_m].vertex[1].v;
			glTexCoord2f(u_m,v_m); glVertex3f(x_m,y_m,z_m);
			
			x_m = sector1.triangle[loop_m].vertex[2].x;
			y_m = sector1.triangle[loop_m].vertex[2].y;
			z_m = sector1.triangle[loop_m].vertex[2].z;
			u_m = sector1.triangle[loop_m].vertex[2].u;
			v_m = sector1.triangle[loop_m].vertex[2].v;
			glTexCoord2f(u_m,v_m); glVertex3f(x_m,y_m,z_m);			
		glEnd();
	}

		
	return TRUE;										// Tutto OK
};

GLvoid GLWVideo::KillGLWindow(GLvoid)								// Distrugge in modo corretto la finestra
{
	if (fullscreen)										// Siamo in modalit Schermo Intero?
	{
		ChangeDisplaySettings(NULL,0);					// Se s torna alla modalit Desktop
		ShowCursor(TRUE);								// Mostra il puntatore del Mouse 
	}

	if (hRC)											// Esiste un Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// E' possibile rilasciare il DC Context e l'RC Context?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// E' possibile cancellare l'RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Setta l'RC a NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// E' possibile rilasciare il DC?
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Setta il DC a NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// E' possibile distruggere la Finestra?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Setta hWnd a NULL
	}

	if (!UnregisterClass("OpenGL",hInstance))			// E' possibile deregistrare la Classe?
	{
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;									// Setta hInstance a NULL
	}
};

/*	Questo codice crea la Finestra OpenGL.  I parametri sono:			     	  *
 *	title			- Titolo della Finestra									      *
 *	width			- Larghezza della Finestra GL o della modalit Schermo Intero *
 *	height			- Altezza della Finestra GL o della modalit Schermo Intero   *
 *	bits			- Numero di Bits da usare per il colore (8/16/24/32)	      *
 *	fullscreenflag	- Usa la modalit Schermo Intero (TRUE) o Finestra (FALSE)    */
 
BOOL GLWVideo::CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Contiene i risultati dopo la ricerca 
	WNDCLASS	wc;						// Struttura Windows Class 
	DWORD		dwExStyle;				// Stile esteso della finestra
	DWORD		dwStyle;				// Stile della finestra 
	RECT		WindowRect;				// Rettangolo della finestra. Valori Superiore Sinistro/Inferiore Destro
	WindowRect.left=(long)0;			// Setta il valore Sinistro a 0
	WindowRect.right=(long)width;		// Setta il valore Destro alla Larghezza richiesta
	WindowRect.top=(long)0;				// Setta il valore Superiore a 0
	WindowRect.bottom=(long)height;		// Setta il valore Inferiore all'Altezza richiesta

	fullscreen=fullscreenflag;			// Setta il flag globale Fullscreen (schermo intero o modalit finestra)

	hInstance			= GetModuleHandle(NULL);				// Tiene l'istanza per la Finestra
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Ridisegna al ridimensionamento, e DC personale per la Finestra.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// Procedure di gestione dei Messaggi (WndProc)
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Setta l'Istanza
	wc.hIcon			= LoadIcon(NULL, /*IDI_ICON1*/IDI_WINLOGO);			// Carica l'icona di Default
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Carica il puntatore del mouse a freccia
	wc.hbrBackground	= NULL/*(HBRUSH)COLOR_BTNSHADOW*/;						// Non  richiesto uno sfondo per GL
	wc.lpszMenuName		= NULL;									// Non vogliamo un menu
	wc.lpszClassName	= "OpenGL";								// Setta il nome della Classe

	if (!RegisterClass(&wc))									// Prova a registrare la Classe Finestra (WC)
	{
		MessageBox(NULL,"Errore nella creazione della finestra.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Restituisce FALSE
	}
	
	if (fullscreen)												// Provare ad attivare la modalit schermo intero?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Si assicura che la memoria sia pulita
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Dimensione della struttura Devmode 
		dmScreenSettings.dmPelsWidth	= width;				// Larghezza dello schermo selezionata
		dmScreenSettings.dmPelsHeight	= height;				// Altezza dello schermo selezionata
		dmScreenSettings.dmBitsPerPel	= bits;					// Bits Per Pixel selezionati
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Prova a settare la modalit selezionata e ottiene i risultati.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// Se la modalit fallisce, offre due opzioni.  Chiudere o usare la modalit finestra.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Seleziona la modalit finestra.  Fullscreen = FALSE
			}
			else
			{
				// Una finestra di messaggio avvisa l'utente che il programma sar chiuso.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Siamo in modalit schermo intero?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Stile esteso della Finestra
		dwStyle=WS_POPUP;										// Stile della Finestra
		ShowCursor(FALSE);										// Nasconde il puntatore del Mouse 
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Stile esteso della Finestra
		dwStyle=WS_OVERLAPPEDWINDOW;							// Stile della Finestra
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);		// Aggiusta la Finestra alla dimensione reale richiesta

	// Crea la Finestra
	if (!(hWnd=CreateWindowEx(	dwExStyle,							// Stile esteso per la Finestra
								"OpenGL",							// Nome della Classe
								title,								// Titolo della Finestra
								dwStyle |							// Stile della Finestra definito
								WS_CLIPSIBLINGS |					// Stile Finestra richiesto
								WS_CLIPCHILDREN,					// Stile Finestra richiesto
								0, 0,								// Posizione della Finestra
								WindowRect.right-WindowRect.left,	// Calcola la Larghezza della Finestra
								WindowRect.bottom-WindowRect.top,	// Calcola l'Altezza della Finestra
								NULL,								// No Parent Window
								NULL,								// No Menu
								hInstance,							// Instance
								NULL)))								// Non passa nulla a WM_CREATE
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd dice a Windows come voi pensate che debba essere
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Dimensione del descrittore del Pixel Format 
		1,											// Numero di versione
		PFD_DRAW_TO_WINDOW |						// Il formato deve supportare Window
		PFD_SUPPORT_OPENGL |						// Il formato deve supportare OpenGL
		PFD_DOUBLEBUFFER,							// Deve supportare il Double Buffering
		PFD_TYPE_RGBA,								// Richiede un formato RGBA 
		bits,										// Seleziona la profondit di colore
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Piano principale di disegno
		0,											// Riservato
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Possiamo ottenere un Device Context?
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Windows pu trovare un Pixel Format che coincide con quello richiesto?
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// E' possibile settare il Pixel Format?
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// E' possibile ottenere un Rendering Context?
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Prova ad attivare il Rendering Context
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Mostra la finestra
	SetForegroundWindow(hWnd);						// Priorit molto alta 
	SetFocus(hWnd);									// Setta l'input focus della tastiera sulla Finestra
	ReSizeGLScene(width, height);					// Setta la prospettiva per lo schermo GL 

	if (!InitGL())									// Inizializza la finestra GL creata
	{
		KillGLWindow();								// Ripristina il Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Restituisce FALSE
	}

	return TRUE;									// Tutto OK
};

int GLWVideo::InitWorld(char *s)
{
	// Crea la Finestra OpenGL 
	if (!CreateGLWindow(s, Screen_X, Screen_Y, Screen_BITS, fullscreen))
	{
		return FALSE;									// Chiude se la Finestra non  stata creata
	}
	Wnd_Name=s;

	return TRUE;
};

int GLWVideo::Disegna()
{
	/* se devo scrivere le fps */	
		if (fpsActive) 
		{
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Pulisce lo schermo e il Depth Buffer
			glLoadIdentity();									// Resetta la Modelview Matrix corrente
			glTranslatef(0.0f,0.0f,-1.0f);	
			glRasterPos2f(-0.45f+0.05f*float(cos(cnt1)), 0.32f*float(sin(cnt1)));		
 			glPrint("fps = %i", fps);	// Scrive il testo GL sullo schermo	
		}		

	if ((active && !DrawGLScene()) || (keys[VK_ESCAPE]))	// Attivo?  E' stato ricevuto un messaggio chiudi?
	{
		return TRUE;							// ESC o DrawGLScene hanno segnalato la chiusura
	}
	else									// Non  il momento di chiudere, quindi aggiorna lo schermo
	{
		SwapBuffers(hDC);					// Scambia i Buffer (Double Buffering)
	/*	if (keys['B'] && !bp)
		{
			bp=TRUE;
			blend=!blend;
			if (!blend)
			{
				glDisable(GL_BLEND);
				glEnable(GL_DEPTH_TEST);
			}
			else
			{
				glEnable(GL_BLEND);
				glDisable(GL_DEPTH_TEST);
			}
		}
		if (!keys['B'])
		{
			bp=FALSE;
		}
	*/	
		if (keys['F'] && !fp)
		{		
			fp=TRUE;

			fpsActive=!fpsActive;
		}

		if (!keys['F'])
		{
			fp=FALSE;			
		}

		if (keys[VK_PRIOR])
		{
			z-=0.02f;
		}

		if (keys[VK_NEXT])
		{
			z+=0.02f;
			xpos -= (float)sin(heading*piover180) * 0.05f / Rallentamento;
		}

		if ((keys[VK_UP]) || (GetKeyState('W') & 0x80))
		{
			if (!(giu>=20)) 
				avanti+=2;
		}

		if ((keys[VK_DOWN]) || (GetKeyState('S') & 0x80))
		{
			if (!(giu>=20)) 
				avanti-=2;
		}

		if ((keys[VK_LEFT]) || (GetKeyState('A') & 0x80)) // non dovr ruotare, ma spostare lateralmente, quando la gestione fps sar funzionale al 99%
		{
			heading += (25.0f/Rallentamento);	
			yrot = heading;
		}

		if ((keys[VK_RIGHT]) || (GetKeyState('D') & 0x80))
		{
			heading -= (25.0f/Rallentamento);
			yrot = heading;
		}		

		if (keys[VK_SPACE])  // il salto: da sistemare
		{
			if (suolo)
			{
				giu=-2000;
				gravita();
			}
		}

		if (keys[VK_PRIOR])
		{
			lookupdown-= 1.0f;
		}

		if (keys[VK_NEXT])
		{
			lookupdown+= 1.0f;
		}

		if (keys[VK_F1])						// E' stato premuto F1?
		{
			keys[VK_F1]=FALSE;					// Se si, mette il tasto a FALSE
			KillGLWindow();						// Distrugge la finestra corrente
			fullscreen=!fullscreen;				// Cambia tra modalit Schermo intero/Finestra
			// Ricrea la Finestra OpenGL
			if (!CreateGLWindow(Wnd_Name,Screen_X,Screen_Y,Screen_BITS,fullscreen))
			{
				return TRUE;						// Chiude se la finestra non  stata creata
			}
		}

		if (keys[VK_F2])						// E' stato premuto F1?
		{
			keys[VK_F2]=FALSE;					// Se si, mette il tasto a FALSE
			ypos+=5;
		}


		/* movimento */
			avanti*=collidono;
			if (collidono==1)  // diminuisco la velocit solo se non rimbalza per una collisione (cos non devrebbe entrare mai nei muri)
			{			
					if (avanti>0) avanti--;
					else if (avanti<0) avanti++;			
			}
			if (avanti>max_avanti) avanti=max_avanti;	     // gestisco la velocit massima
			else if (avanti<-max_avanti) avanti=-max_avanti;
			
			if (avanti!=0)
			{
				xpos -= ((float)sin(heading*piover180) * (avanti*0.08f) / Rallentamento);//*collidono;
				zpos -= ((float)cos(heading*piover180) * (avanti*0.08f) / Rallentamento);//*collidono;
			
				if (avanti>0)
				{			
					if (walkbiasangle >= 359.0f)
					{
						walkbiasangle = 0.0f;
					}
					else
					{
						walkbiasangle+= (10 / 20.0f);
					}			
				}
				else if (avanti<0)
				{				
					if (walkbiasangle <= 1.0f)
					{
						walkbiasangle = 359.0f;
					}
					else
					{
						walkbiasangle-= (10 / 20.0f);
					}					
				}
				
				walkbias = (float)sin(walkbiasangle * piover180)/ (20.0f);
			}
	}

	return FALSE; // il ciclo di gioco deve continuare
};




double GLWVideo::max_vertex(double *v, int n)
{
	double max=v[0];

	for (int i=1; i<n; i++)
		if (v[i]>max) max=v[i];

	return max;
}

double GLWVideo::min_vertex(double *v, int n)
{
	double min=v[0];

	for (int i=1; i<n; i++)
		if (v[i]<min) min=v[i];

	return min;
}

GLvoid GLWVideo::controlla_collisione()
{
	int i, j;	
	double v[3];
	double max, min;
	double n_x=0.12f, n_z=0.12f;  // il limite per evetare il bug grafico dell'attraversamento della parete
	double n_p=0.10f;
	//int pavimento=1;   // dato che le condizioni per la collisione con una base sono diverse
	                  // da quelle negli altri versi, con questo stratagemma faccio tutto in 
	                  // un unico ciclo

	if (giu<-1) giu=-1;

	collidono=1;
	suolo=0;
	for (i=0; i<sector1.numtriangles; i++)
	{									
		for (j=0; j<3; j++) v[j]=sector1.triangle[i].vertex[j].x-xpos;
		
		max=max_vertex(v, 3);
		min=min_vertex(v, 3);
		if ( ((max>=-n_x)&&(max<=n_x))||((min>=-n_x)&&(min<=n_x))||((max>=n_x)&&(min<=-n_x)) ) // collisione
		{
			//if (!((max_vertex(v, 3)>=0.5f)&&(min_vertex(v, 3)<=-0.5f)))   // pavimento

			for (j=0; j<3; j++) v[j]=sector1.triangle[i].vertex[j].z-zpos;
			
			max=max_vertex(v, 3);
			min=min_vertex(v, 3);
			if ( ((max>=-n_z)&&(max<=n_z))||((min>=-n_z)&&(min<=n_z))||((max>=n_z)&&(min<=-n_z)) ) // collisione
			{		
				/* in collisione */
					for (j=0; j<3; j++) v[j]=sector1.triangle[i].vertex[j].y-/*walkbias-*/ypos;//-0.25f;
					
					max=max_vertex(v, 3);
					min=min_vertex(v, 3);
					if ( ((max>=0)&&(max<=n_p))||((min>=0)&&(min<=n_p))||((max>=n_p)&&(min<=0)) ) // collisione
					{
						// Collidono ***************
						//Beep(100,1);
						collidono=-1;
					}
				/* pavimento */
					//if ((max_vertex(v, 3)<=0.0f)&&(min_vertex(v, 3)>=-0.25f)) // se lo tocco
					if ( ((max>=-0.10f)&&(max<=0.10f))||((min>=-0.10f)&&(min<=0.10f))||((max>=0.10)&&(min<=-0.10)) ) // collisione
					{	
						suolo=1;
						// da modificare: calcolo della pendenza
						if ((max<=0.5f)&&((keys[VK_UP])||(keys[VK_DOWN])||(GetKeyState('S') & 0x80)||(GetKeyState('W') & 0x80))) giu=-20;  // se  un ostacolo superabile
						else	giu=-vel_gra;		// -1 perch alla fine del ciclo giu dovr essere 0, se il personaggio tocca terra
						//Beep(100,1);

						/* controllo vittoria */
						if (sector1.triangle[i].n_texture==texture_vittoria)
							vittoria=1;
					}
			}
		}

	}
	giu+=vel_gra;	
}

GLvoid GLWVideo::gravita()
{
	ypos-=((float)giu)/10000;
}

/* testo */
	GLvoid GLWVideo::BuildFont(GLvoid)								// Costruisce il nostro Font Bitmap 
	{
		HFONT	font;										// Windows Font ID
		HFONT	oldfont;									// Usato per tenere il vecchio font

		base = glGenLists(96);								// Spazio per 96 caratteri

		font = CreateFont(	-24,							// Altezza del Font
							0,								// Larghezza del Font
							0,								// Angolo di Scappamento
							0,								// Angolo d'Orientamento
							FW_BOLD,						// Peso del Font 
							FALSE,							// Corsivo
							FALSE,							// Sottolineato
							FALSE,							// Barrato
							ANSI_CHARSET,					// Identificatore del Set di Caratteri
							OUT_TT_PRECIS,					// Precisione di Output 
							CLIP_DEFAULT_PRECIS,			// Precisione di Clipping 
							ANTIALIASED_QUALITY,			// Qualit di Output 
							FF_DONTCARE|DEFAULT_PITCH,		// Famiglia e Passo
							"Courier New");					// Nome del Font 

		oldfont = (HFONT)SelectObject(hDC, font);           // Seleziona il Font desiderato
		wglUseFontBitmaps(hDC, 32, 96, base);				// Costruisce 96 caratteri partendo dal carattere 32
		SelectObject(hDC, oldfont);							// Seleziona il Font desiderato
		DeleteObject(font);									// Cancella il Font
	}

	GLvoid GLWVideo::KillFont(GLvoid)									// Cancella le Font List
	{
		glDeleteLists(base, 96);							// Cancella tutti i 96 caratteri
	}

	GLvoid GLWVideo::glPrint(const char *fmt, ...)			// Funzione GL "Print" personalizzata
	{
		char		text[256];								// Contiene la nostra stringa
		va_list		ap;										// Puntatore alla lista degli argomenti

		if (fmt == NULL)									// Se non c' testo
			return;											// Ritorna senza fare niente

		va_start(ap, fmt);									// Analizza la stringa per le variabili
			vsprintf(text, fmt, ap);						// E converte i simboli in numeri
		va_end(ap);											// Il risultato viene messo in "Text"

		glPushAttrib(GL_LIST_BIT);							// Spinge i Bit della Display List 
		glListBase(base - 32);								// Setta la base dei caratteri a 32
		glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Disegna il testo con le Display List 
		glPopAttrib();										// Rimette i settaggi a posto
	}
		
	GLvoid GLWVideo::scrivi(const char *fmt, ...)
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Pulisce lo schermo e il Depth Buffer
		glLoadIdentity();									// Resetta la Modelview Matrix corrente
		glTranslatef(0.0f,0.0f,-1.0f);		
		glRasterPos2f(-0.45f+0.05f*float(cos(cnt1)), 0.32f*float(sin(cnt1)));		
 		glPrint(fmt);	// Scrive il testo GL sullo schermo	
	}


